@@ -41,7 +41,7 @@ import java.util.Properties;
  * connection-accepting server.
  *
  * @author Gennis Emerson
- * @version $Revision: 1.34 $
+ * @version $Revision: 1.35 $
  */
 /* The handleMessage methods may throw only SQLExceptions and 
  * RuntimeExceptions, the first if they can recover, the second
@@ -678,15 +678,16 @@ class Connection implements Runnable
                 {
                     int type = rsmd.getColumnType(i + 1);
                     Debug.getLogWriter(Debug.TEDIOUS).println(mThreadId + 
-                        "getting column " + (i + 1) + "; type " + type);
+                        "getting column " + (i + 1) + "/" + 
+                        rsmd.getColumnName(i + 1) + "; type " + type);
                     switch (type)
                     {
                     case Types.BINARY: 
                     case Types.VARBINARY: 
                         row[i] = rs.getBytes(i + 1);
                         break;
+
                     case Types.LONGVARBINARY: 
-                    case Types.BLOB: 
                         if (longReadLen == 0)
                             row[i] = null;
                         else
@@ -695,26 +696,80 @@ class Connection implements Runnable
                                 longReadLen, longTruncOk);
                         }
                         break;
+
+                    case Types.BLOB: 
+                        if (longReadLen == 0)
+                            row[i] = null;
+                        else
+                        {
+                            Blob blob = rs.getBlob(i + 1); 
+                            if (blob == null)
+                                row[i] = null;
+                            else
+                            {
+                                row[i] = mReadLong(i + 1, 
+                                    blob.getBinaryStream(), longReadLen, 
+                                    longTruncOk);
+                            }
+                        }
+                        break;
+
+                        // The JDBC spec says to prefer
+                        // getCharacterStream for
+                        // LONGVARCHAR. However, getString is
+                        // also supported. We could use getString
+                        // here instead, or create a parameter
+                        // which lets a caller specify that
+                        // getString should be used.
                     case Types.LONGVARCHAR:
+                        if (longReadLen == 0)
+                            row[i] = null;
+                        else
+                        {
+                            char[] chars = mReadLong(i + 1, 
+                                rs.getCharacterStream(i + 1), longReadLen, 
+                                longTruncOk);
+                            row[i] = (chars == null) ? 
+                                null : new String(chars); 
+                        }
+                        break;
+
                     case Types.CLOB: 
                         if (longReadLen == 0)
                             row[i] = null;
                         else
                         {
-                            byte[] bytes = mReadLong(i + 1, 
-                                rs.getUnicodeStream(i + 1), longReadLen, 
-                                longTruncOk);
-                            row[i] = (bytes == null) ? 
-                                null : new String(bytes, "UTF8");
+                            Clob clob = rs.getClob(i + 1);
+                            if (clob == null)
+                                row[i] = null;
+                            else
+                            {
+                                char[] chars = mReadLong(i + 1, 
+                                    clob.getCharacterStream(), longReadLen, 
+                                    longTruncOk);
+                                row[i] = (chars == null) ? 
+                                    null : new String(chars);
+                            }
                         }
                         break;
+
+                        // The driver I mostly use returns arrays
+                        // in a convenient string format. This
+                        // may be updated later to get an Array
+                        // object, build a string, and allow
+                        // callers to specify a term separator.
+                    case Types.ARRAY: 
+                        row[i] = rs.getString(i + 1);
+                        break;
+
                     case Types.CHAR: 
                         if (chopBlanks)
                         {
                             row[i] = mChopBlanks(rs.getString(i + 1));
                             break;
                         }
-                        // Fall through
+                        // Fall through when chopBlanks is not set.
+
                     default:  // This will include any Types.OTHER columns.
                         row[i] = rs.getString(i + 1);
                         break;
@@ -1469,6 +1524,8 @@ class Connection implements Runnable
         int aLongReadLen, boolean aLongTruncOk) 
         throws DbdException, IOException
     {
+        if (aLongReadLen == 0)
+            return null;
         if (anInputStream == null)
             return null;
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
@@ -1494,6 +1551,51 @@ class Connection implements Runnable
         return baos.toByteArray();
     }
 
+
+    /**
+     * Reads a LONG field. Implements the DBI semantics associated with
+     * the LongTruncOk and LongReadLen properties.
+     *
+     * @param aColumnIndex the column index being read, for error reporting
+     * @param aReader a reader returned by a getXXXStream method
+     * @param aLongReadLen the LongReadLen property for this statement
+     * @param aLongTruncOk the LongTruncOk property for this statement
+     * @return a char array read from the stream, or null if the input
+     *      stream is null
+     * @exception DataTruncation if the data is truncated
+     * @exception IOException if an error occurs reading from the stream
+     */
+    private char[] mReadLong(int aColumnIndex, Reader aReader, 
+        int aLongReadLen, boolean aLongTruncOk) 
+        throws DbdException, IOException
+    {
+        if (aLongReadLen == 0)
+            return null;
+        if (aReader == null)
+            return null;
+        CharArrayWriter out = new CharArrayWriter(aLongReadLen); 
+        char[] buffer = new char[sLONG_READ_BUFFER_SIZE]; 
+        int read;
+        int totalread = 0;
+        while (totalread < aLongReadLen &&
+            (read = aReader.read(buffer, 0, 
+            Math.min(buffer.length, (aLongReadLen - totalread)))) != -1)
+        {
+            totalread += read;
+            out.write(buffer, 0, read);
+        }
+        out.close();
+        Debug.getLogWriter(Debug.VERBOSE).println(mThreadId +
+            "read " + totalread + " bytes from LONG column " + aColumnIndex);
+        if (!aLongTruncOk && aReader.read() != -1)
+        {
+            aReader.close();
+            throw new DbdException(DbdException.gDATA_TRUNCATION);
+        }
+        aReader.close();
+        return out.toCharArray();
+    }
+
     /** 
      * Sends the given error information to the client.
      *
